import router, { resetRouter } from './router'
import Layout from '@/layout'
import { checkToken, refreshToken } from '@/api/system'
import { getToken, setToken, deleteToken } from './utils/auth'
import store from './store'
import Vue from 'vue'

// 检测单点登录
const url = decodeURIComponent(window.location.href)
const hasEqual = url.split('res=')[1] ? url.split('res=')[1] : ''
const obj = hasEqual ? JSON.parse(hasEqual) : null
if (obj && obj.data.loginType === 1) {
  // store.commit('user/SET_SINGLE_SIGN_ON', 1)
  store.commit('user/SET_USERINFO', obj.data.user)
  // 单点登录清空路由重新获取权限
  store.commit('menu/SET_ROLES', [])
  setToken(obj.data.token)
}

let isRefresh = true

// 路由拦截
router.beforeEach((to, from, next) => {
  // 无需登录无需获取动态路由
  if (to?.meta?.noLogin) {
    next()
    return
  }
  // 前往登陆页面判断
  if (to.path === '/login-local') {
    if (!getToken()) {
      next()
      return
    }
    onFilterRoutes(to, next)
    next('/')
    return
  }

  if (to.path === '/login') {
    next('/home')
    return
  }
  // 开发环境不进行单点登录
  if (isRefresh) {
    refreshToken().then(({ data }) => {
      setToken(data)
      store.commit('menu/SET_ROLES', []) // 清空路由
    }).catch(() => {
      Vue.prototype.$snackbar.error('刷新token异常')
    }).finally(() => {
      isRefresh = false
      // 校验失败
      next({ ...to, replace: true })
      // deleteToken()
    })
    return
  }

  checkToken().then(() => {
    onFilterRoutes(to, next)
  }).catch(error => {
    deleteToken()
    if (!error?.data?.redirectUrl) {
      Vue.prototype.$snackbar.error(error.message)
      return
    }
    // 本地返回
    if (error.data.redirectUrl === 'local') {
      setTimeout(() => {
        next(`/login-local?redirect=${to.path}`)
      }, 1000)
      return
    }
    window.location.href = error.data.redirectUrl
  })
})
router.afterEach((to, from) => {
  document.title = to.meta.title ?? Vue.prototype.$DEFAULT_TITLE
})
// 路由过滤和跳转
function onFilterRoutes (to, next) {
  if (store.getters.roles && store.getters.roles.length) {
    initRoutes(to, next, store.getters.roles)
    return
  }
  store.dispatch('menu/getMenu2').then(res => {
    initRoutes(to, next, res)
  }).catch(err => {
    deleteToken()
    next({ ...to, replace: true })
    // next('/login-local')
    Vue.prototype.$snackbar.error(err)
  })
}

function initRoutes (to, next, e) {
  // 判断是否刷新页面重新加载路由
  if (!store.getters.refresh) {
    next()
    return
  }
  store.commit('menu/SET_REFRESH', false) // 关闭更新
  // 清除路由
  resetRouter()
  const routes = mapASyncRoutes(e) // 路由替换
  if (!routes.length) {
    deleteToken()
    // next('/login-local')
    next({ ...to, replace: true })
    return
  }
  routes.forEach(item => {
    router.addRoute(item)
  })
  const hasHome = routes.find(_e => _e.name === 'home')
  router.addRoute({ path: '/', redirect: hasHome ? '/home' : chooseFirst(routes[0]) })
  router.addRoute({ path: '*', redirect: '/404', hidden: true })
  next({ ...to, replace: true })
}
function chooseFirst (item) {
  if (item.children && item.children.length > 0 && item.component.name === 'layoutIndex') {
    return chooseFirst(item.children[0])
  }
  return item.path
}
// 递归路由 转换为组件对象和路径
function mapASyncRoutes (data) {
  return data.map(item => {
    item.component = item.component === 'Layout' ? Layout : loadView(item.component)
    if (item.children && item.children.length > 0) {
      item.children = mapASyncRoutes(item.children)
    }
    return item
  })
}

// 路由插件
function loadView (view) {
  return !view ? { render: (c) => c('router-view') } : () => import(`@/views/${view}`)
}
